@@ -60,10 +60,6 @@
 
 /* FIXME: which systems lack this? */
 #define HAVE_GETTIMEOFDAY
-/* FIXME: disabled because it does not parse addresses with a netmask length.
- * The code using it needs to be either fixed or removed.
-#define HAVE_INET_PTON
-*/
 
 /*
  * Please send patches to correctly ignore old releases which lack a RNG
