@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2010 by Marco d'Itri <md@linux.it>.
+ * Copyright (C) 1999-2018 Marco d'Itri <md@linux.it>.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -368,6 +368,12 @@ int handle_query(const char *hserver, const char *hport,
 	    server = guess_server(p);
 	    free(p);
 	    goto retry;
+	case 0x0D:
+	    p = convert_in6arpa(query);
+	    free(server);
+	    server = guess_server(p);
+	    free(p);
+	    goto retry;
 	default:
 	    break;
     }
@@ -555,7 +561,7 @@ char *guess_server(const char *s)
 	char *server = malloc(strlen("whois.nic.") + strlen(tld) + 1);
 	strcpy(server, "whois.nic.");
 	strcat(server, tld);
-	return(server);
+	return server;
     }
 
     /* no dot but hyphen */
@@ -935,7 +941,7 @@ int openconn(const char *server, const char *port)
      * instead of connecting to the actual whois server.
      */
     if (AFL_MODE)
-	return (dup(0));
+	return dup(0);
 
     alarm(60);
 
@@ -1001,7 +1007,7 @@ int connect_with_timeout(int fd, const struct sockaddr *addr,
     struct timeval tv;
 
     if (timeout <= 0)
-	return (connect(fd, addr, addrlen));
+	return connect(fd, addr, addrlen);
 
     if ((savedflags = fcntl(fd, F_GETFL, 0)) < 0)
 	return -1;
@@ -1065,7 +1071,8 @@ void NORETURN sighandler(int signum)
     err_quit(_("Interrupted by signal %d..."), signum);
 }
 
-int japanese_locale(void) {
+int japanese_locale(void)
+{
     char *lang;
 
     lang = getenv("LC_MESSAGE");
@@ -1212,7 +1219,8 @@ char *normalize_domain(const char *dom)
 
 /* server and port have to be freed by the caller */
 void split_server_port(const char *const input,
-	char **server, char **port) {
+	char **server, char **port)
+{
     char *p;
 
     if (*input == '[' && (p = strchr(input, ']'))) {	/* IPv6 */
@@ -1326,6 +1334,69 @@ char *convert_inaddr(const char *s)
     return new;
 }
 
+char *convert_in6arpa(const char *s)
+{
+    char *ip, *p;
+    int character = 0;
+    int digits = 1;
+
+    ip = malloc(40);
+
+    p = strstr(s, ".ip6.arpa");
+    if (!p || p == s) {
+	ip[character] = '\0';
+	return ip;
+    }
+
+    /* start from the first character before ".ip6.arpa" */
+    p--;
+
+    while (1) {
+	/* check that this is a valid digit for an IPv6 address */
+	if (!((*p >= '0' && *p <= '9') || (*p >= 'a' && *p <= 'f') ||
+	      (*p >= 'A' && *p <= 'F'))) {
+	    free(ip);
+	    ip[character] = '\0';
+	    return ip;
+	}
+
+	/* copy the digit to the IP address */
+	ip[character++] = *p;
+
+	/* stop if we have reached the beginning of the string */
+	if (p == s)
+	    break;
+
+	/* stop if we have parsed a complete address */
+	if (character == 39)
+	    break;
+
+	/* add the colon separator every four digits */
+	if ((digits++ % 4) == 0)
+	    ip[character++] = ':';
+
+	/* go to the precedent character and abort if it is not a dot */
+	p--;
+	if (*p != '.') {
+	    ip[character] = '\0';
+	    return ip;
+	}
+
+	/* abort if the string starts with the dot */
+	if (p == s) {
+	    ip[character] = '\0';
+	    return ip;
+	}
+
+	/* go to the precedent character and continue */
+	p--;
+    }
+
+    /* terminate the string */
+    ip[character] = '\0';
+    return ip;
+}
+
 unsigned long myinet_aton(const char *s)
 {
     unsigned long a, b, c, d;
@@ -1356,7 +1427,8 @@ unsigned long asn32_to_long(const char *s)
     return (a << 16) + b;
 }
 
-int isasciidigit(const char c) {
+int isasciidigit(const char c)
+{
     return (c >= '0' && c <= '9') ? 1 : 0;
 }
 
